from .byte import convert_binary_to_byte_seq, convert_binary_to_byte_seq_parallel
from .image import convert_binary_to_image, convert_binary_to_image_parallel, resize_image, resize_image_parallel
from .utils import (
    build_srcs_dsts,
    categorize_folders,
    convert_bytes_to_binary,
    convert_bytes_to_binary_parallel,
    split_labels,
)

__all__ = [
    "convert_binary_to_byte_seq",
    "convert_binary_to_byte_seq_parallel",
    "convert_binary_to_image",
    "convert_binary_to_image_parallel",
    "resize_image",
    "resize_image_parallel",
    "categorize_folders",
    "split_labels",
    "build_srcs_dsts",
    "convert_bytes_to_binary",
    "convert_bytes_to_binary_parallel",
]

__version__ = "0.6.1"



from typing import Optional

import torch
import torch.nn as nn


class MalConv(nn.Module):
    """The MalConv model.

    References:
        - Edward Raff et al. 2018. Malware Detection by Eating a Whole EXE.
          https://arxiv.org/abs/1710.09435
    """

    def __init__(self, num_classes: int = 2, *, num_embeddings: int = 257, padding_idx: Optional[int] = 256) -> None:
        super().__init__()

        # By default, num_embeddings (257) = byte (0-255) + padding (256).
        self.embedding = nn.Embedding(num_embeddings, 8, padding_idx=padding_idx)

        self.conv1 = nn.Conv1d(8, 128, kernel_size=500, stride=500)
        self.conv2 = nn.Conv1d(8, 128, kernel_size=500, stride=500)
        self.relu = nn.ReLU(inplace=True)

        self.maxpool = nn.AdaptiveMaxPool1d(1)

        self.fc = nn.Sequential(
            nn.Linear(128, 128),
            nn.ReLU(inplace=True),
            nn.Linear(128, num_classes),
        )

    def forward(self, x: torch.Tensor) -> torch.Tensor:
        x = self.embedding(x)

        # Treat embedding dimension as channel.
        x = x.permute(0, 2, 1)

        # Perform gated convolution.
        x = self.conv1(x) * torch.sigmoid(self.conv2(x))
        x = self.relu(x)

        x = self.maxpool(x)

        x = self.fc(torch.flatten(x, 1))

        return x



from typing import Any, Callable, Iterable, Optional

from joblib import Parallel, delayed

from ._typing import FilePath


def execute_parallel(
    function: Callable[[FilePath, FilePath], None],
    srcs: Iterable[FilePath],
    dsts: Iterable[FilePath],
    *,
    n_jobs: Optional[int] = None,
    **kwargs: Any,
) -> None:
    """Execute function in parallel."""

    delayed_function = delayed(function)
    Parallel(n_jobs=n_jobs, **kwargs)(delayed_function(src, dst) for src, dst in zip(srcs, dsts))



import functools
import gzip
from typing import Any, Iterable, Optional

import numpy as np

from ._parallel import execute_parallel
from ._typing import FilePath

__all__ = ["convert_binary_to_byte_seq", "convert_binary_to_byte_seq_parallel"]


def convert_binary_to_byte_seq(
    binary_file: FilePath,
    byte_seq_file: FilePath,
    *,
    length: int,
    padding: int = 256,
    compresslevel: int = 6,
) -> None:
    """Convert binary file to byte sequence file. (deprecated)"""

    with open(binary_file, "rb") as f:
        buffer = f.read(length)

    # Since PyTorch embedding layer requires int32/int64 as input, we have to
    # save each byte (plus padding) as int32 instead of int16.
    byte_seq: np.ndarray = np.frombuffer(buffer, dtype=np.uint8)
    byte_seq = byte_seq.astype(dtype=np.int32)

    padding_length = length - len(byte_seq)
    if padding_length > 0:
        padding_seq = np.full(padding_length, padding, dtype=np.int32)
        byte_seq = np.concatenate([byte_seq, padding_seq])

    # Compress byte sequence via gzip.
    with gzip.open(byte_seq_file, "wb", compresslevel=compresslevel) as f:
        f.write(byte_seq.tobytes())


def convert_binary_to_byte_seq_parallel(
    binary_files: Iterable[FilePath],
    byte_seq_files: Iterable[FilePath],
    *,
    length: int,
    padding: int = 256,
    n_jobs: Optional[int] = None,
    **kwargs: Any,
) -> None:
    """Convert binary file to byte sequence file in parallel. (deprecated)"""

    function = functools.partial(convert_binary_to_byte_seq, length=length, padding=padding)
    execute_parallel(function, binary_files, byte_seq_files, n_jobs=n_jobs, **kwargs)
